#pragma once

#include <map>
#include "sequenceUtil.hpp"




class DeBruijnKmer {
    
public:

    DeBruijnKmer(kmer_int_type_t k);
    DeBruijnKmer(const DeBruijnKmer& dk);
    
    unsigned int getID() const;
    kmer_int_type_t get_kmer_int_val() const;
    
    vector<kmer_int_type_t> get_prev_kmers(unsigned int kmer_length);
    vector<kmer_int_type_t> get_next_kmers(unsigned int kmer_length);
    
    void add_prev_kmer(kmer_int_type_t k, unsigned int kmer_length);
    void add_next_kmer(kmer_int_type_t k, unsigned int kmer_length);

    //private:
    
    static unsigned int id_counter;
    unsigned int id;
    kmer_int_type_t _kmer ;
    
    char _prev; // bit array GATC indicating prev kmers
    char _next; // ditto for next kmers
    
    static const char _G_mask; // = 8;
    static const char _A_mask; // = 4;
    static const char _T_mask; // = 2;
    static const char _C_mask; // = 1;
    

};


typedef map<kmer_int_type_t,DeBruijnKmer> DeBruijnKmerMap;


class DeBruijnGraph {

public:

    DeBruijnGraph(unsigned int kmer_length);

    void add_sequence(const string& sequence);
    
    string toString();
    
    string toChrysalisFormat(int component_id, bool sStrand);
    
    vector<kmer_int_type_t> get_root_kmers();
    

private:

    unsigned int _kmer_length;
    
    map<kmer_int_type_t,DeBruijnKmer> _kmer_map;
    
    DeBruijnKmer& get_kmer_node(kmer_int_type_t t);
    
    vector<DeBruijnKmer> deconvolute_DS_mirror_graph();
    
    
};
